/* A simple UDisk emulation
   It uses libusb so most parts of the Host UDisk funktions will be
   emulated on a PC. The code and the import lib uses Borland Tools.
   You have to change this for other compilers (see #pragma link...)

   Usage
     - copy a file to a FAT32 formated stick
     - the file has to be les than 64kbyte and must be in the rootdir
     - bind that stick to libusb using zadig
     - setup VID / PID of that stick in MY_USB_VID and MY_USB_PID
     - setup Fname to the filename used just use 8.3 format
     
   Note
     - that stick wont get a drivenumber anymore unless its unbound
       with zadig. This code uses cpp because otherwise libusb.h throws
       errors. Anyway its still pure c and because of Keil C51 C89.

   Changes
     - MaxLun omited because memory sticks usually dont use that
     - since just SP_READ10 is used here bulkonly cmd is done
       directly in UDisk_ReadOneSector()

    On a 8 bit compiler it might be more eficient if not using memcpy()
    but copy manually in a loop.
*/
#pragma  link "libusb.lib" // link against the lib this is borland specific

#include <stdio.h>
#include <stdint.h>

#include "libusb.h"
#include "..\inc\fat32.h"        // offsets for the fat entries  
#include "..\inc\usb\usb11.h"    // my std usb def file 
#include "..\inc\usb\usbmsd10.h" // some definitions for Memory Class Devices 
#include "..\inc\ch559usb.h"     // some extra structs from WCH
#include "..\inc\wcherror.h"     // error codes

// remove the keil specific mem declarators
#define xdata
#define code
#define data
#define idata

#define MY_USB_VID   0x090C // change this according to your device
#define MY_USB_PID   0x1000
#define SECTOR_SIZE  512    // it will work for 512 byte sectors only
#define BULK_REQSIZE 64     // do the bulk in 64 byte junks even on USB2.0
#define CBW_SIZE     31     // dont use sizeof to avoid allgin problems on PC
#define CBS_SIZE     13

#define pSetupReq ((PUSB_SETUP_REQ)TxBuffer) //map the setup to TxBuffer

struct libusb_device_handle *devh = NULL;
int    Debuglevel = 0;


/************* globals with Keil mspace declarators ********************/
uint8_t code Fname[12] ="CH559UUPHEX"; //8.3 upcase space padded filename
uint8_t code SetupGetDevDescr [8] = { 0x80, 0x06, 0x00, 0x01, 0x00, 0x00, 0x12, 0x00 };
uint8_t code SetupGetCfgDescr [8] = { 0x80, 0x06, 0x00, 0x02, 0x00, 0x00, 0x04, 0x00 };
uint8_t code SetupSetUsbAddr  [8] = { 0x00, 0x05, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00 };
uint8_t code SetupSetUsbConfig[8] = { 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
//uint8_t code GetMaxLUN      [8] = { 0xA1, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00 };
//uint8_t code SetReset       [8] = { 0xA1, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };

// we are just reading so cmd is enough
uint8_t code SPC_RD_Cmd[31] =
{
  'U','S','B','C', 0,0,1,0, //signature + tag
  SECTOR_SIZE & 0xFF,SECTOR_SIZE >> 8, 0, 0, //len =512 bytes
  0x80,                    // flag
  0x00,                    // lun
  0x0A,                    // cblen
  SPC_CMD_READ10,0, 0,0,0,0, 0,0,1,0,
  0,0, 0,0,0,0
//7,5,0x81,2,0x40,00
};

uint8_t xdata RxBuffer[64];
uint8_t xdata TxBuffer[512];
uint8_t xdata BootSect[512];

uint8_t data  HostDeviceStatus;
uint8_t data  DeviceEp0Size;
uint8_t data  DevEndpOUTAddr;
uint8_t data  DevEndpINAddr;
//uint8_t data  CurrentLun;

uint32_t data PartStart;     // sector where partition start
uint32_t data PartDataStart; // sector where data starts
uint32_t data Rootcluster;   // cluster for the root directory
uint16_t data FileSize;      // Filesize of hardcoded file
uint32_t data FileStart;     // sector where the binary starts
uint8_t  data MaxCluster;

/********************* simiulation code **************************/
/* report a sector buffer or parts of it */
void PrintSector(uint8_t *buffer,int lines)
{
   int i,j;
   for (i=0;i<lines;i++)
   {
      printf("%4.4x : ",i*0x10);
      for (j=0;j<16;j++) printf("%2.2X ",buffer[j+i*0x10]);
      for (j=0;j<16;j++)
      {
         if ((buffer[j+i*0x10] >0x20)&&(buffer[j+i*0x10] <0x80))
           printf("%c",    buffer[j+i*0x10]);
         else printf(".");
      }
      printf ("\n");
   }
}

// replacement for lowlevel.a51
// on a pc just return the value lsbfirst
uint16_t SwapReadWord(uint8_t xdata *buf,uint16_t offset)
{
   return (*((uint16_t*) &(buf)[(offset)]));
}

uint32_t SwapReadLong(uint8_t xdata *buf,uint16_t offset)
{
   return (*((uint32_t*) &(buf)[(offset)]));
}

/* delete a 1k page in flash */
uint8_t IAP_DeletePage(uint8_t page)
{
   printf("Page deleted 0x%2.2X\n",page);
   return 0;  // this is just a simulation;
}

/* flash one word */
uint8_t ProgWord(uint16_t Addr, uint16_t Data)
{
   if ((Addr % 32)==0) printf("Addr: 0x%4.4X\n",Addr); //debug
   return 0;  // this is just a simulation;
}

/* try to open the libusb device */
int SearchForDevice(uint32_t Vendor,uint32_t Product)
{
   int rc = libusb_init(NULL);
   uint8_t buffer[100];
   if (rc < 0)
   {
      fprintf(stderr, "Error initializing libusb: %s\n", libusb_error_name(rc));
      return -1;
   }
   // Set debugging output to max level. */
   libusb_set_debug(NULL, Debuglevel);

   /* Look for a specific device and open it. */
   devh = libusb_open_device_with_vid_pid (NULL, Vendor, Product);
   if (!devh)
   {
      fprintf(stderr, "Can't find the requested device!\n");
      fprintf(stderr, "use zadig to connect a device to libusb!\n");
      return -1;
   }
   libusb_claim_interface(devh, 0);
   return 0;
}
/* do a bulk out */
uint8_t Host_BulkOutTransfer(uint8_t *buffer, uint8_t  len, uint16_t timeout)
{
   int transf;
   int status = libusb_bulk_transfer (devh,
                                      DevEndpOUTAddr,
                                      buffer,
                                      (int)len,
                                      &transf,
                                      timeout);
   if ((status != 0) | (transf != len))
   {
      fprintf(stderr, "Transfer failed! status: %d transf: %d\n",status,transf);
      return ERR_USB_TRANSFER | DevEndpOUTAddr;
   }
   return 0;
}
/* do a bulk in */
uint8_t Host_BulkInTransfer(uint8_t *buffer, uint8_t  len, uint16_t timeout)
{
   int status = libusb_bulk_transfer (devh,
                                      DevEndpINAddr|0x80,
                                      buffer,
                                      (int)len,
                                      NULL,
                                      timeout);
   if (status != 0)
   {
      fprintf(stderr, "Transfer failed! status: %d \n",status);
      return ERR_USB_TRANSFER | DevEndpINAddr;
   }
   return 0;
}

uint8_t Host_CtrlTransfer(uint8_t xdata *buffer, uint8_t idata *Retlen)
{
   int result;
   uint16_t wLength = pSetupReq->wLengthH<<8 | pSetupReq->wLengthL;
   result = libusb_control_transfer (devh,
                                     pSetupReq->bRequestType,
                                     pSetupReq->bRequest,
                                     pSetupReq->wValueH<<8 | pSetupReq->wValueL,
                                     pSetupReq->wIndexH<<8 | pSetupReq->wIndexL,
                                     buffer,
                                     pSetupReq->wLengthH<<8 | pSetupReq->wLengthL,
                                     1000
                                    );
   if (result < wLength)
   {
      fprintf(stderr, "CtrlTransfer failed!\n");
      return ERR_USB_BUF_OVER;
   }
   if(Retlen) *Retlen = result;
   return ERR_SUCCESS;
}

/******************* common code for C51 and PC ************************/
/* program one sector into code flash note the extra param index */
uint8_t IAP_Prom(uint8_t xdata *buffer,uint16_t len,uint8_t index)
{
   uint16_t i;
   uint8_t  error;
   uint16_t Addr;
   uint16_t Data;
   // just delete the neccasary pages on the fly
   if ((index & 0x01)==0) IAP_DeletePage(index >>1);
   // sector to Address
   Addr = index << 9;
   // flash one sector
   for(i=0;i<len;i+=2)
   {
      //Data = (buffer[2*i] | (UINT16)buffer[2*i+1]<<8);//todo is there a better way?
      Data = buffer[2*i+1];
      Data = Data << 8 | buffer[2*i];
      error = ProgWord( Addr,Data );
      Addr+=2;
      if( error != 0x00 )break;
   }
   return error;
}

/* compare code with root filename */
uint8_t strComp(uint8_t code *source, uint8_t xdata *target, uint8_t len)
{
  uint8_t i;
  for(i=0;i<len;i++)
  {
    if(*source++ != *target++)  return 1;
  }
  return 0;
}

/* Read a secor using bulk only cmd */
uint8_t UDisk_ReadOneSector(uint8_t *DataBuf, uint32_t SectorNo)
{
   uint8_t  error;
   CBW      cbw;
   uint16_t remLen;
   uint8_t  i;
   //
   //send the cmd
   //
   memcpy (&cbw,SPC_RD_Cmd,CBW_SIZE);
// cbw.CBW_Lun   = CurrentLun;
// cbw.CBW_Cb[1] = CurrentLun<<4;
#ifdef __C51__
   (uint32_t)cbw.CBW_Cb[2]= SectorNo;
#else
   cbw.CBW_Cb[2] = (UINT8)(SectorNo>>24); //msb first
   cbw.CBW_Cb[3] = (UINT8)(SectorNo>>16);
   cbw.CBW_Cb[4] = (UINT8)(SectorNo>>8);
   cbw.CBW_Cb[5] = (UINT8)SectorNo;
#endif
   // todo this will be wrong on a Keil C51 cbw has the wrong mem spec
   error = Host_BulkOutTransfer((uint8_t *)&cbw,CBW_SIZE,1000);
   if (error) return error;

   //remLen = (((uint16_t)cbw.CBW_Len[1] <<8) |cbw.CBW_Len[0]);
   remLen = cbw.CBW_Len[1];
   remLen = (remLen << 8) | cbw.CBW_Len[0];
   //printf("remLen %d\n",remLen);
   if (remLen)
   {
      //if(cbw.CBW_Flags) // not neccesary we dont write
      {
         while(remLen)
         {
            error = Host_BulkInTransfer(RxBuffer,BULK_REQSIZE,1000);
            if (error != ERR_SUCCESS) return error;
            memcpy(DataBuf,RxBuffer,BULK_REQSIZE);
            remLen  -= BULK_REQSIZE;
            DataBuf += BULK_REQSIZE;
         }
      }
      // read CSW status
      error = Host_BulkInTransfer(RxBuffer,CBS_SIZE,1000);
      if ( error != ERR_SUCCESS ) return error;
      //printf("CSW: ");
      //for (i=0;i< 0x0d;i++) printf("%2.2X ",RxBuffer[i]);
      //printf("\n");
   }
   return RxBuffer[0x0c]; //status byte
}

/*read the bootsector */
uint8_t UDisk_ReadDiskDBR(void)
{
   uint8_t  error;
   uint8_t  partitions;
   uint16_t offset;
   uint32_t sector;
   //
   // read the Masterbootrecord /sector 0
   //
   error = UDisk_ReadOneSector(BootSect, 0L);
   if(error) return error;
   if (SwapReadWord(BootSect,wSignature)!=0xAA55) return ERR_MBR_ERROR;
   //
   // find the partitiontable and read the bootrecord
   //
   offset = PartTableStart;
   partitions = 4;
   while(partitions)
   {
      sector = SwapReadLong(BootSect,offset+lFirstSector); //Part Start
      if (BootSect[offset+bActiveFlag ]==0x80)
      {
         //printf("BootSector: %8.8X\n",sector);
         error = UDisk_ReadOneSector(BootSect, sector);
         if (error) return error;
         break;
      }
      partitions --;
      offset +=16;
   }
   if(partitions == 0) return ERR_PART_ERROR; //no active partition found
   //
   // here the first active bootsector is loaded
   //
   if (SwapReadWord(BootSect,wSignature)!=0xAA55) return ERR_FAT_ERROR;
   if (SwapReadWord(BootSect,wBytesPerSector) != SECTOR_SIZE) return ERR_FAT_ERROR;
   if( (BootSect[0]==0xEB) &&  // JMP
       (BootSect[1]==0x58) &&  // Marker FAT32
       (BootSect[2]==0x90))    // JMP offset
   {
      PartStart = sector;
      sector   +=  SwapReadWord(BootSect,wReservedSectors);
      sector   += (SwapReadLong(BootSect,lSectorsPerFat) * BootSect[bNumberOfFats]);
      PartDataStart = sector;
      Rootcluster   = SwapReadLong(BootSect,lRootCluster);
      MaxCluster    = BootSect[bSectorsPerCLuster];
      return 0;
   }
   return ERR_FAT_ERROR; // not a FAT32 partiton
}

/* get a new cluster from the FAT for file read */
uint32_t UDisk_ReadDiskFAT(uint8_t xdata *Buffer, uint32_t ClusNum)
{
   uint8_t  result;
   uint16_t j;//j=0;
   uint32_t sectorNumber;
   uint32_t nextClusNum;
   //
   // calculate the sector where the FAT starts
   //
   //sectorNumber  = (ClusNum << 2) / SECTOR_SIZE; //original
   sectorNumber  = ClusNum >> 7;                   // optimized
   sectorNumber += PartStart;
   sectorNumber += SwapReadWord(BootSect,wReservedSectors);
   //printf("FAT Sector %u\n",sectorNumber);
   result = UDisk_ReadOneSector(Buffer,sectorNumber);
   if(result == ERR_SUCCESS)
   {
      j = (ClusNum*4)%(SECTOR_SIZE);  // orginal todo is there a better way?
      nextClusNum = SwapReadLong(Buffer,j);
   }
   if( nextClusNum == 0x0fffffff) return 0; //no more file clusters
   return nextClusNum;
}

/*serch for the hard coded filename in the rootdir */
uint8_t UDisk_FileOpen(uint8_t xdata *Buffer)
{
   uint8_t   result;
   uint8_t   j;
   uint16_t  i;
   uint32_t  nextClusNum;
   uint32_t  sectnum;


   nextClusNum = Rootcluster;
   do
   {
      sectnum  = MaxCluster*(nextClusNum-Rootcluster);
      sectnum += PartDataStart;
      for(j=0; j < MaxCluster; j++)
      {
         //printf("FileOpen:ClusterSect: 0x%8.8X\n",sectnum+j);
         result = UDisk_ReadOneSector(Buffer,sectnum+j);
         if(result == 0/* ERR_SUCCESS*/)
         {
            //PrintSector(Buffer,32);
            //printf("\n\n");
            i = 0; //find the filename
            while (strComp(Fname, &Buffer[i],11) && (i<512))
            {
               i += DIR_ENTRY_SIZE;
            }

            if(i < SECTOR_SIZE)
            {
               //just allow files < 64k
               if (SwapReadWord(Buffer,i+lFileSize+2) !=0) return ERR_TOO_LARGE;
               FileSize  = SwapReadWord(Buffer,i+lFileSize);
               // a hack to avoid 32 bit shift which is costly on c51
               FileStart = SwapReadLong(Buffer,i+wFirstClusterHi-2);
               FileStart &=0xFFFF0000;      // trow the lowpart
               FileStart |=SwapReadWord(Buffer,i+wFirstClusterLo);

               // some debug autput
               printf("Filename  : ");
               for (j=0;j<8;j++) printf("%c",Buffer[i+j]);
               printf(".");
               for (j=8;j<11;j++) printf("%c",Buffer[i+j]);
               printf("\n");
               printf("FileStart : 0x%8.8X\n",FileStart);
               printf("FileSize  : %d bytes\n",FileSize);

               return ERR_SUCCESS;
            }
            else  result = ERR_MISS_FILE;
         }
      }
      nextClusNum = UDisk_ReadDiskFAT(Buffer,nextClusNum);
   } while (nextClusNum);
   return result;
}

/*read the file and flash it*/
uint8_t UDisk_ReadFile(uint8_t xdata *Buffer)
{
   uint8_t   error, j;
   uint8_t   index;
   uint16_t  len;
   uint32_t  nextClusNum;
   uint32_t  secnum;
   uint32_t  resvsize;

   resvsize    = FileSize;
   nextClusNum = FileStart;
   //printf("StartCl: 0x%8.8X\n",nextClusNum);
   index = 0;
   do
   {
      secnum  = MaxCluster*(nextClusNum-Rootcluster);
      secnum += PartDataStart;

      for(j=0; j < MaxCluster; j++)
      {
         //printf("Sector : 0x%8.8X\n",secnum+j);
         error = UDisk_ReadOneSector(Buffer,secnum+j);
         if(error == ERR_SUCCESS)
         {
            len = (resvsize > SECTOR_SIZE) ? SECTOR_SIZE
                                           : resvsize;
            IAP_Prom(Buffer, len,index);
            PrintSector(Buffer,32);
            index ++;
            resvsize -= len;
            if(!resvsize) return error;
         }
      }
      nextClusNum = UDisk_ReadDiskFAT(Buffer,nextClusNum);
   } while(nextClusNum);
   return error;
}

/* enum requests */
uint8_t Host_CtrlGetDeviceDescr( void )
{
   uint8_t error;
   uint8_t len;
   uint8_t buffer[100];
   int result;

   memcpy(pSetupReq,SetupGetDevDescr,sizeof(USB_SETUP_REQ));
/*
     printf("Setup: ");
     for (error=0;error<8;error++) printf("0x%2.2X ",TxBuffer[error]);
     printf("\n");
*/
   error = Host_CtrlTransfer(TxBuffer, &len);  // perform the request

   DeviceEp0Size = ( (pDeviceDescriptor)TxBuffer ) -> bMaxPacketSize0;
   return error;
}

uint8_t Host_CtrlSetUsbAddress(uint8_t addr)
{
   uint8_t error;
   memcpy(pSetupReq,SetupSetUsbAddr,sizeof(USB_SETUP_REQ));
/*
     printf("Setup: ");
     for (error=0;error<8;error++) printf("0x%2.2X ",TxBuffer[error]);
     printf("\n");
*/
   pSetupReq -> wValueL = addr;
/*  dont do is on the PC
    error = Host_CtrlTransfer( NULL, NULL);
    if (error) return error;
    Host_SetUsbAddr(addr);
    mDelaymS( 10 );
*/
    return( ERR_SUCCESS );
}

uint8_t Host_CtrlSetUsbConfig(uint8_t cfg)
{
   uint8_t error;
   memcpy(pSetupReq,SetupSetUsbConfig,sizeof(USB_SETUP_REQ));
/*
     printf("Setup: ");
     for (error=0;error<8;error++) printf("0x%2.2X ",TxBuffer[error]);
     printf("\n");
*/
   pSetupReq -> wValueL = cfg;
   error = Host_CtrlTransfer(NULL, NULL);
   return error;
}

uint8_t Host_CtrlGetConfigDescr( void )
{
   uint8_t error;
   uint8_t len;

   memcpy(pSetupReq,SetupGetCfgDescr,sizeof(USB_SETUP_REQ));
/*
     printf("Setup: ");
     for (error=0;error<8;error++) printf("0x%2.2X ",TxBuffer[error]);
     printf("\n");
*/
   error = Host_CtrlTransfer( TxBuffer, &len );
   if (error) return error;
   if ( len < ( (PUSB_SETUP_REQ)SetupGetCfgDescr ) -> wLengthL )
   {
      fprintf(stderr, "GetConfig failed!\n");
      return( ERR_USB_BUF_OVER );
   }
   len = ( (pConfigurationDescriptor)TxBuffer ) -> wTotalLengthLo;
   if ( len > 0x40/*MAX_PACKET_SIZE*/ )
   {
      fprintf(stderr, "CtrlTransfer size too large %d!\n",len);
      return ERR_USB_BUF_OVER ;
   }

   memcpy(pSetupReq,SetupGetCfgDescr,sizeof(USB_SETUP_REQ));
/*
     printf("Setup: ");
     for (error=0;error<8;error++) printf("0x%2.2X ",TxBuffer[error]);
     printf("\n");
*/
   pSetupReq -> wLengthL = len;
   error = Host_CtrlTransfer( TxBuffer, &len );
   if (error) return error;
   if (len < ((PUSB_SETUP_REQ)SetupGetCfgDescr) -> wLengthL
      || len < ((pConfigurationDescriptor)TxBuffer) -> wTotalLengthLo)
      return( ERR_USB_BUF_OVER );

   Host_CtrlSetUsbConfig(( (pConfigurationDescriptor)TxBuffer ) -> bConfigurationValue);

   if((((PUSB_CFG_DESCR_LONG)TxBuffer) -> endp_descr[0].bEndPointAddress)>0x80)
   {
      DevEndpOUTAddr = (((PUSB_CFG_DESCR_LONG)TxBuffer) -> endp_descr[1].bEndPointAddress)&0x0f;
      DevEndpINAddr  = (((PUSB_CFG_DESCR_LONG)TxBuffer) -> endp_descr[0].bEndPointAddress)&0x0f;
   }
   else
   {
      DevEndpOUTAddr = (((PUSB_CFG_DESCR_LONG)TxBuffer) -> endp_descr[0].bEndPointAddress)&0x0f;
      DevEndpINAddr  = (((PUSB_CFG_DESCR_LONG)TxBuffer) -> endp_descr[1].bEndPointAddress)&0x0f;
   }
   return( ERR_SUCCESS );
}

/*
uint8_t Host_CtrlGetMaxLUN( void )
{
   uint8_t  error;
   uint8_t  len;

   memcpy(pSetupReq,GetMaxLUN,sizeof(USB_SETUP_REQ));
   //CopySetupReqPkg( GetMaxLUN );

     printf("Setup: ");
     for (error=0;error<8;error++) printf("0x%2.2X ",TxBuffer[error]);
     printf("\n");

   error = Host_CtrlTransfer(TxBuffer, &len);
   if ( error != ERR_SUCCESS ) return error;
   if ( len < ( (PUSB_SETUP_REQ)GetMaxLUN ) -> wLengthL ) return ERR_USB_BUF_OVER;
   CurrentLun = TxBuffer[0];
   //UH_RX_CTRL = UH_TX_CTRL = bUH_R_TOG | bUH_T_TOG;
   return ERR_SUCCESS;
}
*/

uint8_t InitRootDevice( void )  //
{
   uint8_t  i;
   uint8_t  error;
   uint8_t  dv_cls;
   uint8_t  if_cls;
/*
   ResetRootHubPort( );
   for (i = 0, s = 0; i < 100; i ++)
   {
      mDelaymS(1);
      if ( EnableRootHubPort( ) == ERR_SUCCESS )
      {
         i = 0;
         s ++;
         if ( s > 100 ) break;
      }
   }
   if (i)
   {
      DisableRootHubPort( );
      return (ERR_USB_DISCON);
   }
   SetUsbSpeed( 1 );
   //
   //  enum the stick
   //
*/
   error = Host_CtrlGetDeviceDescr();
   if (error == ERR_SUCCESS )
   {
      dv_cls = ((pDeviceDescriptor)TxBuffer) -> bDeviceClass;
      error = Host_CtrlSetUsbAddress( ( (PUSB_SETUP_REQ)SetupSetUsbAddr ) -> wValueL );  // USB豸ַ
      if (error == ERR_SUCCESS)
      {
         error = Host_CtrlGetConfigDescr();
         if (error == ERR_SUCCESS)
         {
            if_cls = ( (PUSB_CFG_DESCR_LONG)TxBuffer ) -> itf_descr.bInterfaceClass;  // ӿ
            if (dv_cls == 0x00 && if_cls == USB_CLASS_MSD)
            {
               HostDeviceStatus = DISK_USB_ADDR;
               return ERR_SUCCESS;
            }
            else
            {
               return( ERR_USB_UNSUPPORT );
            }
         }
      }
   }
   HostDeviceStatus = DISK_CONNECT;
   return error;
}

void main (void)
{
   uint8_t error;
   uint8_t i;
   uint8_t buf[512];
   if (SearchForDevice(MY_USB_VID,MY_USB_PID)) return;
   error = InitRootDevice();
   if (error) fprintf(stderr, "enum failure!\n");
   else
   {
      HostDeviceStatus = DISK_USB_ADDR;
/*
      //todo why do they send this 10 times?
      for ( i = 0; i != 10; i ++ )
      {
         error = Host_CtrlGetMaxLUN();
         if (error) fprintf(stderr, "lun failed! %d\n",i);
      }
*/
      error = UDisk_ReadDiskDBR();
      if (error) printf("DBR ErrCode 0x%2X\n",error);
      else
      {
         // search for the file
         error = UDisk_FileOpen(buf);
         if (error) printf("FileOpen ErrCode 0x%2X\n",error);
         else  error=UDisk_ReadFile(buf);
      }
   }
   if(devh) libusb_close(devh);
}